"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const crypto_1 = __importDefault(require("crypto"));
const qs_1 = __importDefault(require("qs"));
const axios_1 = __importDefault(require("axios"));
const types_1 = require("../../types");
exports.generateSignature = (data, path, secret, nonce) => {
    const secretBuffer = new Buffer(secret, 'base64');
    const hash = crypto_1.default.createHash('sha256');
    const hmac = crypto_1.default.createHmac('sha512', secretBuffer);
    const hashDigest = hash.update(nonce + data).digest('binary');
    const hmacDigest = hmac
        .update(path + hashDigest, 'binary')
        .digest('base64');
    return hmacDigest;
};
exports.getHttpPrefix = (network) => {
    switch (network) {
        case types_1.Network.MAINNET:
            return 'https://api.kraken.com';
        default:
            throw new Error('Kraken only supports the MAINNET network.');
    }
};
const prefix = 'https://api.kraken.com';
exports.privateRequest = (method, params = {}, auth) => __awaiter(this, void 0, void 0, function* () {
    const path = `/0/private/${method}`;
    const nonce = Date.now() * 1000;
    const data = qs_1.default.stringify(Object.assign({ nonce }, params));
    const key = auth.key;
    const secret = auth.secret;
    const signature = exports.generateSignature(data, path, secret, nonce);
    try {
        const response = yield axios_1.default.post(`${prefix}${path}`, data, {
            headers: {
                'API-Key': key,
                'API-Sign': signature,
            },
        });
        return response;
    }
    catch (e) {
        const error = new Error('Failed to execute request on exchange.');
        error.original = e;
        throw error;
    }
});
