"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const axios_1 = __importDefault(require("axios"));
const types_1 = require("../../../types");
const common_1 = require("./common");
const wethToEth_1 = require("../../../utils/wethToEth");
const getHttpUrl = (options) => {
    const base = wethToEth_1.wethToEth(options.base);
    const quote = wethToEth_1.wethToEth(options.quote);
    switch (options.network) {
        case types_1.Network.MAINNET:
            return `https://api.kraken.com/0/public/Depth?pair=${base}${quote}`;
        default:
            throw new Error('Kraken only supports the MAINNET network.');
    }
};
exports.fetch = (options) => __awaiter(this, void 0, void 0, function* () {
    const url = getHttpUrl(options);
    const response = yield axios_1.default
        .get(url)
        .then(value => value.data);
    if (response.error && response.error.length) {
        throw new Error(response.error.join(''));
    }
    const result = response.result;
    const base = options.base;
    const quote = options.quote;
    const key = `X${base}X${quote}`;
    const orderbook = (result && result[key]) || {
        asks: [],
        bids: [],
    };
    const asks = orderbook.asks.map(order => common_1.normalizeOrder(options, types_1.AskOrBid.ASK, order));
    const bids = orderbook.bids.map(order => common_1.normalizeOrder(options, types_1.AskOrBid.BID, order));
    return [].concat(asks, bids);
});
