"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const Rx = __importStar(require("rxjs"));
const operators_1 = require("rxjs/operators");
const webSocket_1 = require("rxjs/webSocket");
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
const types_1 = require("../../../types");
const debug = __importStar(require("../../../debug"));
const cleanEvents_1 = require("../../../utils/cleanEvents");
const wethToEth_1 = require("../../../utils/wethToEth");
const common_1 = require("./common");
const token_math_1 = require("@melonproject/token-math");
const subscribeMessage = (options) => {
    const base = wethToEth_1.wethToEth(options.base);
    const quote = wethToEth_1.wethToEth(options.quote);
    const message = {
        event: 'subscribe',
        pair: [`${base}/${quote}`],
        subscription: {
            name: 'book',
        },
    };
    return message;
};
const getWebsocketUrl = (options) => {
    switch (options.network) {
        case types_1.Network.MAINNET:
            return 'wss://ws.kraken.com';
        default:
            throw new Error('Invalid network.');
    }
};
const normalizeUpdateEvent = (options, message) => {
    const bids = !!message.b
        ? message.b.map(order => common_1.normalizeOrder(options, types_1.AskOrBid.BID, order))
        : [];
    const asks = !!message.a
        ? message.a.map(order => common_1.normalizeOrder(options, types_1.AskOrBid.ASK, order))
        : [];
    return [].concat(bids, asks).map(order => {
        if (token_math_1.isZero(order.trade.base.quantity)) {
            return {
                id: order.id,
                event: types_1.NormalizedMessageType.REMOVE,
                exchange: types_1.Exchange.KRAKEN,
                network: options.network,
                base: options.base,
                quote: options.quote,
            };
        }
        return {
            order,
            id: order.id,
            event: types_1.NormalizedMessageType.SET,
            exchange: types_1.Exchange.KRAKEN,
            network: options.network,
            base: options.base,
            quote: options.quote,
        };
    });
};
const normalizeSnapshotEvent = (options, message) => {
    const bids = !!message.bs
        ? message.bs.map(order => common_1.normalizeOrder(options, types_1.AskOrBid.BID, order))
        : [];
    const asks = !!message.as
        ? message.as.map(order => common_1.normalizeOrder(options, types_1.AskOrBid.ASK, order))
        : [];
    return {
        event: types_1.NormalizedMessageType.SNAPSHOT,
        exchange: types_1.Exchange.KRAKEN,
        network: options.network,
        base: options.base,
        quote: options.quote,
        orders: [].concat(bids, asks),
    };
};
exports.watch = (options) => {
    const ws$ = new Rx.Observable(observer => {
        const open$ = new Rx.Subject();
        const close$ = new Rx.Subject();
        const ws$ = webSocket_1.webSocket({
            WebSocketCtor: isomorphic_ws_1.default,
            closeObserver: close$,
            openObserver: open$,
            url: getWebsocketUrl(options),
        });
        ws$.next(subscribeMessage(options));
        open$.subscribe(() => {
            debug.log('Opening connection.');
        });
        close$.subscribe(() => {
            debug.log('Closing connection.');
        });
        return ws$.subscribe(observer);
    }).pipe(operators_1.share());
    const messages$ = ws$.pipe(operators_1.filter(R.is(Array)), operators_1.map(R.nth(1)));
    const snapshots$ = messages$.pipe(operators_1.filter(R.cond([[R.has('as'), R.T], [R.has('bs'), R.T], [R.T, R.F]])), operators_1.map(message => normalizeSnapshotEvent(options, message)));
    const updates$ = messages$.pipe(operators_1.filter(R.cond([[R.has('a'), R.T], [R.has('b'), R.T], [R.T, R.F]])), operators_1.flatMap(message => normalizeUpdateEvent(options, message)));
    return Rx.merge(snapshots$, updates$).pipe(operators_1.tap(event => debug.log('Source event: %e', event)), cleanEvents_1.cleanEvents(), operators_1.tap(event => debug.log('Output event: %e', event)));
};
