"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const Rx = __importStar(require("rxjs"));
const R = __importStar(require("ramda"));
const operators_1 = require("rxjs/operators");
const webSocket_1 = require("rxjs/webSocket");
const isomorphic_ws_1 = __importDefault(require("isomorphic-ws"));
const types_1 = require("../../../types");
const debug = __importStar(require("../../../debug"));
const wethToEth_1 = require("../../../utils/wethToEth");
const subscribeMessage = (options) => {
    const base = wethToEth_1.wethToEth(options.base);
    const quote = wethToEth_1.wethToEth(options.quote);
    const message = {
        event: 'subscribe',
        pair: [`${base}/${quote}`],
        subscription: {
            name: 'trade',
        },
    };
    return message;
};
const getWebsocketUrl = (options) => {
    switch (options.network) {
        case types_1.Network.MAINNET:
            return 'wss://ws.kraken.com';
        default:
            throw new Error('Invalid network.');
    }
};
exports.feed = (options) => {
    const ws$ = new Rx.Observable(observer => {
        const open$ = new Rx.Subject();
        const close$ = new Rx.Subject();
        const ws$ = webSocket_1.webSocket({
            WebSocketCtor: isomorphic_ws_1.default,
            closeObserver: close$,
            openObserver: open$,
            url: getWebsocketUrl(options),
        });
        ws$.next(subscribeMessage(options));
        open$.subscribe(() => {
            debug.log('Opening connection.');
        });
        close$.subscribe(() => {
            debug.log('Closing connection.');
        });
        return ws$.subscribe(observer);
    }).pipe(operators_1.share());
    const messages$ = ws$.pipe(operators_1.filter(R.is(Array)), operators_1.flatMap(trades => (trades && trades[1]) || []), operators_1.map(trade => ({
        base: options.base,
        quote: options.quote,
        exchange: types_1.Exchange.KRAKEN,
        network: types_1.Network.MAINNET,
        price: `${trade[0]}`,
        volume: `${trade[1]}`,
        time: parseInt(`${trade[2]}`, 10),
        side: trade[3] === 'b' ? types_1.BuyOrSell.BUY : types_1.BuyOrSell.SELL,
        type: trade[4] === 'm' ? types_1.OrderType.MARKET : types_1.OrderType.LIMIT,
    })));
    return messages$;
};
