"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const R = __importStar(require("ramda"));
const types_1 = require("../../types");
const protocol_1 = require("@melonproject/protocol");
const token_math_1 = require("@melonproject/token-math");
const proxyPath = [
    'thirdPartyContracts',
    'exchanges',
    'kyber',
    'kyberNetworkProxy',
];
exports.fetch = (options) => __awaiter(this, void 0, void 0, function* () {
    const quantities = options.quantities || [1, 10, 100, 1000];
    const environment = options.environment;
    const deployment = environment.deployment;
    const proxy = R.path(proxyPath, deployment);
    const quote = protocol_1.getTokenBySymbol(environment, options.quote);
    const base = protocol_1.getTokenBySymbol(environment, options.base);
    const bidQuantities = quantities.map(quantity => {
        return token_math_1.createQuantity(quote, quantity);
    });
    const bidsPromise = Promise.all(bidQuantities.map(quantity => {
        return protocol_1.getExpectedRate(environment, proxy, {
            makerAsset: quote,
            takerAsset: base,
            fillTakerQuantity: quantity,
        });
    })).then(response => response.filter((price) => {
        return !token_math_1.isZero(price.quote.quantity);
    }));
    const askQuantities = quantities.map(quantity => {
        return token_math_1.createQuantity(base, quantity);
    });
    const asksPromise = Promise.all(askQuantities.map(quantity => {
        return protocol_1.getExpectedRate(environment, proxy, {
            makerAsset: base,
            takerAsset: quote,
            fillTakerQuantity: quantity,
        });
    })).then(response => response.filter((price) => {
        return !token_math_1.isZero(price.quote.quantity);
    }));
    const [bidsResponse, asksResponse] = yield Promise.all([
        bidsPromise,
        asksPromise,
    ]);
    const bids = bidsResponse.map((bid, index) => {
        const volume = quantities[index];
        const base = bid.base;
        const quote = bid.quote;
        const trade = token_math_1.createPrice(token_math_1.createQuantity(base.token, token_math_1.multiply(base.quantity, volume)), token_math_1.createQuantity(quote.token, token_math_1.multiply(quote.quantity, volume)));
        const pair = `${base.token.address}:${quote.token.address}`;
        const key = `${types_1.Exchange.KYBER_NETWORK}:${pair}:${volume}`;
        const id = Buffer.from(key).toString('base64');
        return {
            id,
            exchange: types_1.Exchange.KYBER_NETWORK,
            type: types_1.AskOrBid.BID,
            trade,
        };
    });
    const asks = asksResponse.map((ask, index) => {
        const volume = quantities[index];
        const normalized = token_math_1.normalize(token_math_1.createPrice(ask.quote, ask.base));
        const base = normalized.base;
        const quote = normalized.quote;
        const trade = token_math_1.createPrice(token_math_1.createQuantity(base.token, token_math_1.multiply(base.quantity, volume)), token_math_1.createQuantity(quote.token, token_math_1.multiply(quote.quantity, volume)));
        const pair = `${ask.base.token.address}:${ask.quote.token.address}`;
        const key = `${types_1.Exchange.KYBER_NETWORK}:${pair}:${volume}`;
        const id = Buffer.from(key).toString('base64');
        return {
            id,
            exchange: types_1.Exchange.KYBER_NETWORK,
            type: types_1.AskOrBid.ASK,
            trade,
        };
    });
    return [].concat(bids, asks);
});
